#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_webhooks_payload_templates
short_description: Resource module for networks _webhooks _payload _templates
description:
  - Manage operations create, update and delete of the resource networks _webhooks
    _payload _templates.
  - Create a webhook payload template for a network. - > Destroy a webhook payload
    template for a network. Does not work for included templates 'wpt_00001', 'wpt_00002',
    'wpt_00003', 'wpt_00004', 'wpt_00005', 'wpt_00006', 'wpt_00007' or 'wpt_00008'
    .
  - Update a webhook payload template for a network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  body:
    description: The liquid template used for the body of the webhook message. Either
      `body` or `bodyFile` must be specified.
    type: str
  bodyFile:
    description: A Base64 encoded file containing liquid template used for the body
      of the webhook message. Either `body` or `bodyFile` must be specified.
    type: str
  headers:
    description: The liquid template used with the webhook headers.
    elements: dict
    suboptions:
      name:
        description: The name of the header template.
        type: str
      template:
        description: The liquid template for the headers.
        type: str
    type: list
  headersFile:
    description: A Base64 encoded file containing the liquid template used with the
      webhook headers.
    type: str
  name:
    description: The name of the new template.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  payloadTemplateId:
    description: PayloadTemplateId path parameter. Payload template ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks createNetworkWebhooksPayloadTemplate
    description: Complete reference of the createNetworkWebhooksPayloadTemplate API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-webhooks-payload-template
  - name: Cisco Meraki documentation for networks deleteNetworkWebhooksPayloadTemplate
    description: Complete reference of the deleteNetworkWebhooksPayloadTemplate API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-webhooks-payload-template
  - name: Cisco Meraki documentation for networks updateNetworkWebhooksPayloadTemplate
    description: Complete reference of the updateNetworkWebhooksPayloadTemplate API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-webhooks-payload-template
notes:
  - SDK Method used are
    networks.Networks.create_network_webhooks_payload_template,
    networks.Networks.delete_network_webhooks_payload_template,
    networks.Networks.update_network_webhooks_payload_template,
  - Paths used are
    post /networks/{networkId}/webhooks/payloadTemplates,
    delete /networks/{networkId}/webhooks/payloadTemplates/{payloadTemplateId},
    put /networks/{networkId}/webhooks/payloadTemplates/{payloadTemplateId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_webhooks_payload_templates:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    body: '{"event_type":"{{alertTypeId}}","client_payload":{"text":"{{alertData}}"}}'
    bodyFile: Qm9keSBGaWxl
    headers:
      - name: Authorization
        template: Bearer {{sharedSecret}}
    headersFile: SGVhZGVycyBGaWxl
    name: Custom Template
    networkId: string
- name: Delete by id
  cisco.meraki.networks_webhooks_payload_templates:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    payloadTemplateId: string
- name: Update by id
  cisco.meraki.networks_webhooks_payload_templates:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    body: '{"event_type":"{{alertTypeId}}","client_payload":{"text":"{{alertData}}"}}'
    bodyFile: Qm9keSBGaWxl
    headers:
      - name: Authorization
        template: Bearer {{sharedSecret}}
    headersFile: SGVhZGVycyBGaWxl
    name: Custom Template
    networkId: string
    payloadTemplateId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "payloadTemplateId": "string",
      "type": "string",
      "name": "string",
      "headers": [
        {
          "name": "string",
          "template": "string"
        }
      ],
      "body": "string",
      "sharing": {
        "byNetwork": {
          "adminsCanModify": true
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_pod_settings
version_added: "2.11.0"
short_description: Manage Pod Settings on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Pod Settings on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Fabric Policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the template.
    - The template must be a Fabric Policy template.
    - This parameter or O(template) is required.
    type: str
  name:
    description:
    - The name of the Pod Settings.
    type: str
    aliases: [ pod_settings ]
  uuid:
    description:
    - The UUID of the Pod Settings.
    - This parameter is required when the O(name) needs to be updated.
    type: str
    aliases: [ pod_settings_uuid ]
  description:
    description:
    - The description of the Pod Settings.
    type: str
  macsec_policy:
    description:
    - The MACsec Policy to be used.
    - This parameter is mutually exclusive with O(macsec_policy_uuid).
    - Providing an empty dictionary O(macsec_policy={}) will remove the MACsec Policy from the Pod Settings.
    type: dict
    suboptions:
      name:
        description:
        - The name of the MACsec Policy to be used.
        type: str
  macsec_policy_uuid:
    description:
    - The UUID of the MACsec Policy to be used.
    - Providing an empty string O(macsec_policy_uuid="") will remove the MACsec Policy from the Pod Settings.
    - This parameter is mutually exclusive with O(macsec_policy).
    type: str
  ntp_policy:
    description:
    - The NTP Policy to be used.
    - This parameter is mutually exclusive with O(ntp_policy_uuid).
    - Providing an empty dictionary O(ntp_policy={}) will remove the NTP Policy from the Pod Settings.
    type: dict
    suboptions:
      name:
        description:
        - The name of the NTP Policy to be used.
        type: str
  ntp_policy_uuid:
    description:
    - The UUID of the NTP Policy to be used.
    - Providing an empty string O(ntp_policy_uuid="") will remove the NTP Policy from the Pod Settings.
    - This parameter is mutually exclusive with O(ntp_policy).
    type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Fabric Policy template.
- The O(macsec_policy) or O(macsec_policy_uuid) must exist before using it with this module in your playbook.
  Use M(cisco.mso.ndo_macsec_policy) to create the MACsec Policy.
- The O(ntp_policy) or O(ntp_policy_uuid) must exist before using it with this module in your playbook.
  Use M(cisco.mso.ndo_ntp_policy) to create the NTP Policy.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_macsec_policy
- module: cisco.mso.ndo_ntp_policy
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create Pod Settings
  cisco.mso.ndo_pod_settings:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_settings
    state: present
  register: create_pod_settings

- name: Update Pod Settings
  cisco.mso.ndo_pod_settings:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_settings
    description: Updated Pod Settings
    macsec_policy:
      name: ansible_macsec_policy
    ntp_policy:
      name: ansible_ntp_policy
    state: present

- name: Update the Pod Settings using UUID
  cisco.mso.ndo_pod_settings:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    uuid: "{{ create_pod_settings.current.uuid }}"
    macsec_policy_uuid: "{{ create_macsec_policy.current.uuid }}"
    ntp_policy_uuid: "{{ create_ntp_policy.current.uuid }}"
    state: present

- name: Query the Pod Settings using name
  cisco.mso.ndo_pod_settings:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_settings
    state: query
  register: query_with_name

- name: Query the Pod Settings using UUID
  cisco.mso.ndo_pod_settings:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    uuid: "{{ create_pod_settings.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query all Pod Settings in a template
  cisco.mso.ndo_pod_settings:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    state: query
  register: query_all_objects

- name: Delete the Pod Settings using Name
  cisco.mso.ndo_pod_settings:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test
    name: ansible_pod_settings
    state: absent

- name: Delete the Pod Settings using UUID
  cisco.mso.ndo_pod_settings:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_id: ansible_test
    uuid: "{{ create_pod_settings.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data, check_if_all_elements_are_none
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        name=dict(type="str", aliases=["pod_settings"]),
        uuid=dict(type="str", aliases=["pod_settings_uuid"]),
        description=dict(type="str"),
        macsec_policy=dict(
            type="dict",
            options=dict(
                name=dict(type="str"),
            ),
        ),
        macsec_policy_uuid=dict(type="str"),
        ntp_policy=dict(
            type="dict",
            options=dict(
                name=dict(type="str"),
            ),
        ),
        ntp_policy_uuid=dict(type="str"),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("template", "template_id"),
            ("macsec_policy", "macsec_policy_uuid"),
            ("ntp_policy", "ntp_policy_uuid"),
        ],
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
        required_one_of=[
            ["template", "template_id"],
        ],
    )

    mso = MSOModule(module)

    template_name = module.params.get("template")
    template_id = module.params.get("template_id")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    macsec_policy = module.params.get("macsec_policy")
    if macsec_policy is not None and check_if_all_elements_are_none(macsec_policy.values()):
        macsec_policy = {}
    macsec_policy_uuid = module.params.get("macsec_policy_uuid")
    ntp_policy = module.params.get("ntp_policy")
    if ntp_policy is not None and check_if_all_elements_are_none(ntp_policy.values()):
        ntp_policy = {}
    ntp_policy_uuid = module.params.get("ntp_policy_uuid")
    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "fabric_policy", template_name, template_id)
    mso_template.validate_template("fabricPolicy")
    object_description = "Pod Settings"

    match = mso_template.get_pod_settings_object(uuid, name)

    if (uuid or name) and match:
        set_pod_settings_policy_names(mso_template, match.details)
        mso.existing = mso.previous = copy.deepcopy(match.details)  # Query a specific object
    elif match:
        mso.existing = [set_pod_settings_policy_names(mso_template, obj) for obj in match]  # Query all objects

    pod_settings_path = "/fabricPolicyTemplate/template/podPolicyGroups/{0}".format(match.index if match else "-")

    ops = []

    if state == "present":

        mso_values = {
            "name": name,
            "description": description,
        }

        if macsec_policy or macsec_policy_uuid:
            mso_values["fabricMACsec"] = mso_template.get_macsec_policy_object(
                macsec_policy_uuid,
                macsec_policy.get("name") if macsec_policy else macsec_policy,
                fail_module=True,
            ).details.get("uuid")
        if ntp_policy or ntp_policy_uuid:
            mso_values["ntp"] = mso_template.get_ntp_policy_object(
                ntp_policy_uuid,
                ntp_policy.get("name") if ntp_policy else ntp_policy,
                fail_module=True,
            ).details.get("uuid")

        if match:
            remove_data = []
            unwanted = []
            if macsec_policy == {} or macsec_policy_uuid == "":
                remove_data.append("fabricMACsec")
                unwanted.extend(["fabricMACsec", "fabricMACsecName"])
            if ntp_policy == {} or ntp_policy_uuid == "":
                remove_data.append("ntp")
                unwanted.extend(["ntp", "ntpName"])

            append_update_ops_data(ops, match.details, pod_settings_path, mso_values, remove_data)
            mso.sanitize(mso_values, collate=True, unwanted=unwanted)
            set_pod_settings_policy_names(mso_template, mso.proposed)
        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=pod_settings_path, value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=pod_settings_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = mso_template.get_pod_settings_object(uuid, name, search_object=response)
        if match:
            set_pod_settings_policy_names(mso_template, match.details)
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        if mso.proposed:
            set_pod_settings_policy_names(mso_template, mso.proposed)
        mso.existing = mso.proposed if state == "present" else {}
    mso.exit_json()


def set_pod_settings_policy_names(mso_template, pod_settings):
    pod_settings["templateName"] = mso_template.template_name
    pod_settings["templateId"] = mso_template.template_id
    if pod_settings.get("fabricMACsec"):
        pod_settings["fabricMACsecName"] = mso_template.get_macsec_policy_object(uuid=pod_settings["fabricMACsec"]).details.get("name")
    if pod_settings.get("ntp"):
        pod_settings["ntpName"] = mso_template.get_ntp_policy_object(uuid=pod_settings["ntp"]).details.get("name")
    return pod_settings


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Sabari Jaganathan <sajagana@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}


DOCUMENTATION = r"""
---
module: aci_interface_config
short_description: Manage Interface Configuration of Access (infra:PortConfig) and Fabric (fabric:PortConfig) Ports is only supported for ACI 5.2(7)+
description:
- Manage Interface Configuration of Access (infra:PortConfig) and Fabric (fabric:PortConfig) Ports is only supported for ACI 5.2(7)+
options:
  policy_group:
    description:
    - The name of the Policy Group being associated with the Port.
    - The I(policy_group) and I(breakout) cannot be configured simultaneously.
    type: str
    aliases: [ policy_group_name ]
  breakout:
    description:
    - The Breakout Map of the interface.
    - The I(policy_group) and I(breakout) cannot be configured simultaneously.
    type: str
    choices: [ 100g-2x, 100g-4x, 10g-4x, 25g-4x, 50g-8x ]
  description:
    description:
    - The description of the Interface Configuration object.
    type: str
    aliases: [ descr ]
  node:
    description:
    - The ID of the Node.
    - The value must be between 101 to 4000.
    type: int
    aliases: [ node_id ]
  pc_member:
    description:
    - The name of the Port Channel Member Policy (lacp:IfPol).
    - A Port Channel Member Policy is used to override LACP port priority and transmit rate of LACP packets.
    type: str
    aliases: [ port_channel_member ]
  port_type:
    description:
    - The type of the interface can be either access or fabric.
    type: str
    default: access
    choices: [ access, fabric ]
  role:
    description:
    - The role of the switch (node) can be either a leaf or a spine.
    - The APIC defaults to leaf when unset during creation.
    type: str
    aliases: [ node_type ]
    choices: [ leaf, spine ]
  admin_state:
    description:
    - The Admin State of the Interface.
    - The APIC defaults to up when unset during creation.
    type: str
    choices: [ up, down ]
  interface_type:
    description:
    - The type of the interface.
    type: str
    default: switch_port
    choices: [ switch_port, pc_or_vpc, fc, fc_port_channel, leaf_fabric, spine_access, spine_fabric ]
  interface:
    description:
    - The address of the interface.
    - The format of the interface value should be 1/1/1 (card/port_id/sub_port) or 1/1 (card/port_id).
    - The Card ID must be in the range of 1 to 255.
    - The Port ID must be in the range of 1 to 128.
    - The Sub Port ID must be in the range of 0 to 64.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(infra:PortConfig).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Sabari Jaganathan (@sajagana)
"""
EXAMPLES = r"""
- name: Add an interface with port channel(PC) policy group
  cisco.aci.aci_interface_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    role: leaf
    port_type: access
    interface_type: pc_or_vpc
    policy_group: ans_test_port_channel
    node: 502
    interface: "2/2/2"
    state: present
  delegate_to: localhost

- name: Add an interface with port channel(PC) policy group with a PC member policy override
  cisco.aci.aci_interface_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    role: leaf
    port_type: access
    interface_type: pc_or_vpc
    policy_group: ans_test_port_channel
    pc_member: ans_test_pc_member_policy
    node: 502
    interface: "2/2"
    state: present
  delegate_to: localhost

- name: Breakout the existing interface with "100g-4x"
  cisco.aci.aci_interface_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    role: leaf
    port_type: access
    node: 502
    interface: "2/2/2"
    breakout: "100g-4x"
    state: present
  delegate_to: localhost

- name: Query an access interface with node id
  cisco.aci.aci_interface_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    port_type: access
    node: 201
    state: query
  delegate_to: localhost

- name: Query a fabric interface with node id
  cisco.aci.aci_interface_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    port_type: fabric
    node: 202
    state: query
  delegate_to: localhost

- name: Query all access interfaces
  cisco.aci.aci_interface_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    port_type: access
    state: query
  delegate_to: localhost

- name: Query all fabric interfaces
  cisco.aci.aci_interface_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    port_type: fabric
    state: query
  delegate_to: localhost

- name: Remove a interface
  cisco.aci.aci_interface_config:
    host: apic
    username: admin
    password: SomeSecretPassword
    port_type: access
    node: 201
    interface: "1/1/1"
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec

PORT_TYPE_MAPPING = dict(
    access=dict(
        root_class="infraInfra",
        root_class_rn="infra",
        interface_class="infraPortConfig",
    ),
    fabric=dict(
        root_class="fabricInst",
        root_class_rn="fabric",
        interface_class="fabricPortConfig",
    ),
)

ADMIN_STATE_MAPPING = {"up": "no", "down": "yes"}


POLICY_GROUP_MAPPING = dict(
    switch_port="uni/infra/funcprof/accportgrp-{0}",
    pc_or_vpc="uni/infra/funcprof/accbundle-{0}",
    fc="uni/infra/funcprof/fcaccportgrp-{0}",
    fc_port_channel="uni/infra/funcprof/fcaccbundle-{0}",
    leaf_fabric="uni/fabric/funcprof/leportgrp-{0}",
    spine_access="uni/infra/funcprof/spaccportgrp-{0}",
    spine_fabric="uni/fabric/funcprof/spportgrp-{0}",
)


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        policy_group=dict(type="str", aliases=["policy_group_name"]),
        breakout=dict(type="str", choices=["100g-2x", "100g-4x", "10g-4x", "25g-4x", "50g-8x"]),
        description=dict(type="str", aliases=["descr"]),
        node=dict(type="int", aliases=["node_id"]),
        pc_member=dict(type="str", aliases=["port_channel_member"]),
        port_type=dict(type="str", default="access", choices=["access", "fabric"]),
        role=dict(type="str", choices=["leaf", "spine"], aliases=["node_type"]),
        admin_state=dict(type="str", choices=["up", "down"]),
        interface_type=dict(
            type="str",
            default="switch_port",
            choices=["switch_port", "pc_or_vpc", "fc", "fc_port_channel", "leaf_fabric", "spine_access", "spine_fabric"],
        ),
        interface=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["node", "interface"]],
            ["state", "present", ["node", "interface"]],
        ],
        mutually_exclusive=[("policy_group", "breakout")],
    )

    policy_group = module.params.get("policy_group")
    breakout = module.params.get("breakout")
    description = module.params.get("description")
    node = module.params.get("node")
    pc_member = module.params.get("pc_member")
    port_type = module.params.get("port_type")
    role = module.params.get("role")
    admin_state = module.params.get("admin_state")
    interface = module.params.get("interface")
    interface_type = module.params.get("interface_type")
    state = module.params.get("state")

    aci = ACIModule(module)

    try:
        if node is not None and int(node) not in range(101, 4001):
            aci.fail_json(msg="Node ID: {0} is invalid; it must be in the range of 101 to 4000.".format(node))

        card, port_id, sub_port = (None, None, None)
        if interface is not None:
            interface_parts = interface.split("/")
            if len(interface_parts) == 3:
                card, port_id, sub_port = interface_parts
            elif len(interface_parts) == 2:
                card, port_id = interface_parts
                sub_port = 0
            else:
                aci.fail_json(msg="Interface: {0} is invalid; The format must be either card/port/sub_port(1/1/1) or card/port(1/1)".format(interface))

            if int(card) not in range(1, 256):
                aci.fail_json(msg="Card ID: {0} is invalid; it must be in the range of 1 to 255.".format(card))

            if int(port_id) not in range(1, 129):
                aci.fail_json(msg="Port ID: {0} is invalid; it must be in the range of 1 to 128.".format(port_id))

            # Sub Port ID - 0 is default value
            if int(sub_port) not in range(0, 65):
                aci.fail_json(msg="Sub Port ID: {0} is invalid; it must be in the range of 0 to 64.".format(sub_port))
    except ValueError as error:
        aci.fail_json(msg="Interface configuration failed due to: {0}".format(error))

    root_class = PORT_TYPE_MAPPING.get(port_type)["root_class"]
    root_class_rn = PORT_TYPE_MAPPING.get(port_type)["root_class_rn"]
    interface_class = PORT_TYPE_MAPPING.get(port_type)["interface_class"]

    aci.construct_url(
        root_class=dict(
            aci_class=root_class,
            aci_rn=root_class_rn,
        ),
        subclass_1=dict(
            aci_class=interface_class,
            aci_rn="portconfnode-{0}-card-{1}-port-{2}-sub-{3}".format(node, card, port_id, sub_port),
            target_filter=dict(node=node),
        ),
    )

    aci.get_existing()

    if breakout is None and policy_group:
        policy_group_dn = POLICY_GROUP_MAPPING.get(interface_type).format(policy_group)
    else:
        # To handle the existing object property
        policy_group_dn = ""

    if state == "present":
        aci.payload(
            aci_class=interface_class,
            class_config=dict(
                assocGrp=policy_group_dn,
                brkoutMap=breakout,
                card=card,
                description=description,
                node=node,
                pcMember=pc_member,
                port=port_id,
                role=role,
                shutdown=ADMIN_STATE_MAPPING.get(admin_state),
                subPort=sub_port,
            ),
        )

        aci.get_diff(aci_class=interface_class)

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The nxos_vlans class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
    remove_empties,
    to_list,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.utils.utils import (
    search_obj_in_list,
)


class Vlans(ConfigBase):
    """
    The nxos_vlans class
    """

    gather_subset = ["min"]

    gather_network_resources = ["vlans"]

    def __init__(self, module):
        super(Vlans, self).__init__(module)

    def get_platform(self):
        """Wrapper method for getting platform info
        This method exists solely to allow the unit test framework to mock calls.
        """
        return self.facts.get("ansible_net_platform", "")

    def get_vlans_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        self.facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        vlans_facts = self.facts["ansible_network_resources"].get("vlans")
        if not vlans_facts:
            return []

        return vlans_facts

    def edit_config(self, commands):
        """Wrapper method for `_connection.edit_config()`
        This exists solely to allow the unit test framework to mock device connection calls.
        """
        return self._connection.edit_config(commands)

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        commands = []
        warnings = []

        if self.state in self.ACTION_STATES:
            existing_vlans_facts = self.get_vlans_facts()
            self._platform = self.get_platform()
        else:
            existing_vlans_facts = []
            self._platform = ""

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_vlans_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self.edit_config(commands)
            result["changed"] = True

        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_vlans_facts = self.get_vlans_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_vlans_facts(data=running_config)

        if self.state in self.ACTION_STATES:
            result["before"] = existing_vlans_facts
            if result["changed"]:
                result["after"] = changed_vlans_facts

        elif self.state == "gathered":
            result["gathered"] = changed_vlans_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_vlans_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params.get("config") or []
        have = existing_vlans_facts
        resp = self.set_state(self._sanitize(want), self._sanitize(have))
        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        if state in ("overridden", "merged", "replaced", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(state),
            )

        commands = list()
        if state == "overridden":
            commands.extend(self._state_overridden(want, have))
        elif state == "deleted":
            commands.extend(self._state_deleted(want, have))
        else:
            for w in want:
                if state in ["merged", "rendered"]:
                    commands.extend(self._state_merged(w, have))
                elif state == "replaced":
                    commands.extend(self._state_replaced(w, have))

        return commands

    def remove_default_states(self, obj):
        """Removes non-empty but default states from the obj."""
        default_states = {"enabled": True, "state": "active", "mode": "ce"}
        for k in default_states.keys():
            if obj.get(k) == default_states[k]:
                obj.pop(k, None)
        return obj

    def _state_replaced(self, want, have):
        """The command generator when state is replaced.
        Scope is limited to vlan objects defined in the playbook.
        :rtype: A list
        :returns: The minimum command set required to migrate the current
                  configuration to the desired configuration.
        """
        obj_in_have = search_obj_in_list(want["vlan_id"], have, "vlan_id")
        if obj_in_have:
            # Diff the want and have
            diff = dict_diff(want, obj_in_have)
            # Remove merge items from diff; what's left will be used to
            # remove states not specified in the playbook
            for k in dict(set(want.items()) - set(obj_in_have.items())).keys():
                diff.pop(k, None)
        else:
            diff = want

        # Remove default states from resulting diff
        diff = self.remove_default_states(diff)

        # merged_cmds: 'want' cmds to update 'have' states that don't match
        # replaced_cmds: remaining 'have' cmds that need to be reset to default
        merged_cmds = self.set_commands(want, have)
        replaced_cmds = []
        if obj_in_have:
            # Remaining diff items are used to reset states to default
            replaced_cmds = self.del_attribs(diff)
        cmds = []
        if replaced_cmds or merged_cmds:
            cmds += ["vlan %s" % str(want["vlan_id"])]
            cmds += merged_cmds + replaced_cmds
        return cmds

    def _state_overridden(self, want, have):
        """The command generator when state is overridden.
        Scope includes all vlan objects on the device.
        :rtype: A list
        :returns: the minimum command set required to migrate the current
                  configuration to the desired configuration.
        """
        # overridden behavior is the same as replaced except for scope.
        cmds = []
        existing_vlans = []
        for h in have:
            existing_vlans.append(h["vlan_id"])
            obj_in_want = search_obj_in_list(h["vlan_id"], want, "vlan_id")
            if obj_in_want:
                if h != obj_in_want:
                    replaced_cmds = self._state_replaced(obj_in_want, [h])
                    if replaced_cmds:
                        cmds.extend(replaced_cmds)
            else:
                cmds.append("no vlan %s" % h["vlan_id"])

        # Add wanted vlans that don't exist on the device yet
        for w in want:
            if w["vlan_id"] not in existing_vlans:
                new_vlan = ["vlan %s" % w["vlan_id"]]
                cmds.extend(new_vlan + self.add_commands(w))
        return cmds

    def _state_merged(self, w, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        cmds = self.set_commands(w, have)
        if cmds:
            cmds.insert(0, "vlan %s" % str(w["vlan_id"]))
        return cmds

    def _state_deleted(self, want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        if want:
            for w in want:
                obj_in_have = search_obj_in_list(w["vlan_id"], have, "vlan_id")
                if obj_in_have:
                    commands.append("no vlan " + str(obj_in_have["vlan_id"]))
        else:
            if not have:
                return commands
            for h in have:
                commands.append("no vlan " + str(h["vlan_id"]))
        return commands

    def del_attribs(self, obj):
        """Returns a list of commands to reset states to default"""
        commands = []
        if not obj:
            return commands
        default_cmds = {
            "name": "no name",
            "state": "no state",
            "enabled": "no shutdown",
            "mode": "mode ce",
            "mapped_vni": "no vn-segment",
        }
        for k in obj:
            commands.append(default_cmds[k])
        return commands

    def diff_of_dicts(self, w, obj):
        diff = set(w.items()) - set(obj.items())
        diff = dict(diff)
        if diff and w["vlan_id"] == obj["vlan_id"]:
            diff.update({"vlan_id": w["vlan_id"]})
        return diff

    def add_commands(self, d):
        commands = []
        if not d:
            return commands
        if "name" in d:
            commands.append("name " + d["name"])
        if "state" in d:
            commands.append("state " + d["state"])
        if "enabled" in d:
            if d["enabled"] is True:
                commands.append("no shutdown")
            else:
                commands.append("shutdown")
        if "mode" in d:
            commands.append("mode " + d["mode"])
        if "mapped_vni" in d:
            commands.append("vn-segment %s" % d["mapped_vni"])

        return commands

    def set_commands(self, w, have):
        commands = []
        obj_in_have = search_obj_in_list(w["vlan_id"], have, "vlan_id")
        if not obj_in_have:
            commands = self.add_commands(w)
        else:
            diff = self.diff_of_dicts(w, obj_in_have)
            commands = self.add_commands(diff)
        return commands

    def _sanitize(self, vlans):
        sanitized_vlans = []
        for vlan in vlans:
            if not re.search("N[567][7K]", self._platform):
                if "mode" in vlan:
                    del vlan["mode"]
            sanitized_vlans.append(remove_empties(vlan))
        return sanitized_vlans



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Lukas Holub (@lukasholub)
# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_interface_policy_eigrp
short_description: Manage EIGRP interface policies (eigrp:IfPol)
description:
- Manage EIGRP interface policies for Tenants on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of an existing Tenant.
    type: str
    aliases: [ tenant_name ]
  eigrp:
    description:
    - The EIGRP interface policy name.
    - Note that you cannot change this name after the object has been created.
    type: str
    aliases: [ eigrp_interface, name ]
  bandwidth:
    description:
    - The EIGRP bandwidth in kbps, overrides the bandwidth configured on an interface.
    - This is used to influence path selection.
    - Accepted values range between C(0) and C(2560000000).
    - The APIC defaults to C(0) when unset during creation.
    type: int
  control_state:
    description:
    - The interface policy control state.
    - 'This is a list of one or more of the following controls:'
    - C(bfd) -- Enable Bidirectional Forwarding Detection.
    - C(nexthop_self) -- Nexthop Self.
    - C(split_horizon) -- Split Horizon.
    - C(passive) -- The interface does not participate in the EIGRP protocol and
      will not establish adjacencies or send routing updates.
    - The APIC defaults to C([split_horizon, nexthop_self]) when unset during creation.
    type: list
    elements: str
    choices: [ bfd, nexthop_self, passive, split_horizon ]
  delay:
    description:
    - The EIGRP throughput delay, overrides the delay configured on an interface.
    - This is used to influence path selection.
    - The APIC defaults to C(0) when unset during creation.
    type: int
  delay_unit:
    description:
    - The EIGRP delay units, Wide metrics can use picoseconds accuracy for delay.
    - The APIC defaults to C(tens_of_microseconds) when unset during creation.
    type: str
    choices: [ picoseconds, tens_of_microseconds ]
  hello_interval:
    description:
    - The time interval in seconds between hello packets that EIGRP sends on the interface.
    - The smaller the hello interval, the faster topological changes will be detected, but more routing traffic will ensue.
    - Accepted values range between C(1) and C(65535).
    - The APIC defaults to C(5) when unset during creation.
    type: int
  hold_interval:
    description:
    - The time period of time in seconds before declaring that the neighbor is down.
    - Accepted values range between C(1) and C(65535).
    - The APIC defaults to C(15) when unset during creation.
    type: int
  description:
    description:
    - The description of the EIGRP interface policy.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant) used must exist before using this module in your playbook.
- The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(eigrp:IfPol).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Gaspard Micol (@gmicol)
- Lukas Holub (@lukasholub)
"""

EXAMPLES = r"""
- name: Create an EIGRP interface policy
  cisco.aci.aci_interface_policy_eigrp:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    eigrp: eigrp1
    bandwidth: 10000
    control_state: [split-horizon, nh-self]
    delay: 10
    delay_unit: tens_of_micro
    hello_interval: 5
    hold_interval: 15
    state: present
  delegate_to: localhost

- name: Delete EIGRRP interface policy
  cisco.aci.aci_interface_policy_eigrp:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    eigrp: eigrp1
    state: present
  delegate_to: localhost

- name: Query an EIGRRP interface policy
  cisco.aci.aci_interface_policy_eigrp:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    eigrp: eigrp1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all EIGRP interface policies in tenant production
  cisco.aci.aci_interface_policy_eigrp:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import (
    MATCH_EIGRP_INTERFACE_POLICY_DELAY_UNIT_MAPPING,
    MATCH_EIGRP_INTERFACE_POLICY_CONTROL_STATE_MAPPING,
)


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        eigrp=dict(type="str", aliases=["eigrp_interface", "name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        bandwidth=dict(type="int"),
        control_state=dict(type="list", elements="str", choices=["bfd", "nexthop_self", "passive", "split_horizon"]),
        delay=dict(type="int"),
        delay_unit=dict(type="str", choices=["picoseconds", "tens_of_microseconds"]),
        hello_interval=dict(type="int"),
        hold_interval=dict(type="int"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["eigrp", "tenant"]],
            ["state", "present", ["eigrp", "tenant"]],
        ],
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    eigrp = module.params.get("eigrp")
    delay = module.params.get("delay")
    delay_unit = MATCH_EIGRP_INTERFACE_POLICY_DELAY_UNIT_MAPPING.get(module.params.get("delay_unit"))
    description = module.params.get("description")
    name_alias = module.params.get("name_alias")
    state = module.params.get("state")

    bandwidth = module.params.get("bandwidth")
    if bandwidth is not None and bandwidth not in range(2560000001):
        module.fail_json(msg="Parameter 'bandwidth' is only valid in range between 0 and 2560000000.")

    hello_interval = module.params.get("hello_interval")
    if hello_interval is not None and hello_interval not in range(1, 65536):
        module.fail_json(msg="Parameter 'hello_interval' is only valid in range between 1 and 65535.")

    hold_interval = module.params.get("hold_interval")
    if hold_interval is not None and hold_interval not in range(1, 65536):
        module.fail_json(msg="Parameter 'hold_interval' is only valid in range between 1 and 65535.")

    if module.params.get("control_state"):
        control_state = ",".join([MATCH_EIGRP_INTERFACE_POLICY_CONTROL_STATE_MAPPING.get(v) for v in module.params.get("control_state")])
    else:
        control_state = None

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="eigrpIfPol",
            aci_rn="eigrpIfPol-{0}".format(eigrp),
            module_object=eigrp,
            target_filter={"name": eigrp},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="eigrpIfPol",
            class_config=dict(
                name=eigrp,
                descr=description,
                bw=bandwidth,
                ctrl=control_state,
                delay=delay,
                delayUnit=delay_unit,
                helloIntvl=hello_interval,
                holdIntvl=hold_interval,
                nameAlias=name_alias,
            ),
        )

        aci.get_diff(aci_class="eigrpIfPol")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
filter plugin file for ipaddr filters: ipsubnet
"""
from __future__ import absolute_import, division, print_function

from functools import partial

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddr_utils import (
    _need_netaddr,
    ipaddr,
)


__metaclass__ = type


from ansible.module_utils.common.text.converters import to_text


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment

try:
    import netaddr

    HAS_NETADDR = True
except ImportError:
    # in this case, we'll make the filters return error messages (see bottom)
    HAS_NETADDR = False
else:

    class mac_linux(netaddr.mac_unix):
        pass

    mac_linux.word_fmt = "%.2x"

DOCUMENTATION = """
    name: ipsubnet
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.5.0"
    short_description: This filter can be used to manipulate network subnets in several ways.
    description:
        - This filter can be used to manipulate network subnets in several ways.
    options:
        value:
            description:
            - subnets or individual address or any other values input for ipsubnet plugin
            type: str
            required: True
        query:
            description: |
                You can provide query as 1st argument.
                To check if a given string is a subnet, pass it through the filter without any arguments. If the given
                string is an IP address, it will be converted into a subnet.
                If you specify a subnet size as the first parameter of the ipsubnet() filter, and the subnet size is
                smaller than the current one, you will get the number of subnets a given subnet can be split into.
            type: str
            default: ''
        index:
            description: |
                The second argument of the ipsubnet() filter is an index number; by specifying it you can get a new subnet
                with the specified index.
            type: int
    notes:
"""

EXAMPLES = r"""
#### examples
# Ipsubnet filter plugin with different queries.
vars:
  address: 192.168.144.5
  subnet: 192.168.0.0/16
  ipv6_address: '2001:4860:4860::8888'
  ipv6_subnet: '2600:1f1c:1b3:8f00::/56'
tasks:
  - name: convert IP address to subnet
    debug:
      msg: '{{ address | ansible.utils.ipsubnet }}'
  - name: check if a given string is a subnet
    debug:
      msg: '{{ subnet | ansible.utils.ipsubnet }}'
  - name: Get the number of subnets a given subnet can be split into.
    debug:
      msg: '{{ subnet | ansible.utils.ipsubnet(20) }}'
  - name: Get a 1st subnet
    debug:
      msg: '{{ subnet | ansible.utils.ipsubnet(20, 0) }}'
  - name: Get a last subnet
    debug:
      msg: '{{ subnet | ansible.utils.ipsubnet(20, -1) }}'
  - name: Get first IPv6 subnet that has prefix length /120
    debug:
      msg: '{{ ipv6_subnet | ansible.utils.ipsubnet(120, 0) }}'
  - name: Get last subnet that has prefix length /120
    debug:
      msg: '{{ ipv6_subnet | ansible.utils.ipsubnet(120, -1) }}'
  - name: Get biggest subnet that contains that given IP address.
    debug:
      msg: '{{ address | ansible.utils.ipsubnet(20) }}'
  - name: Get 1st smaller subnet by specifying 0 as index number
    debug:
      msg: '{{ address | ansible.utils.ipsubnet(18, 0) }}'
  - name: Get last subnet
    debug:
      msg: '{{ address | ansible.utils.ipsubnet(18, -1) }}'
  - name: >-
      The rank of the IP in the subnet (the IP is the 36870nth /32 of the
      subnet)
    debug:
      msg: '{{ address | ansible.utils.ipsubnet(subnet) }}'
  - name: The rank in the /24 that contain the address
    debug:
      msg: '{{ address | ansible.utils.ipsubnet(''192.168.144.0/24'') }}'
  - name: An IP with the subnet in the first /30 in a /24
    debug:
      msg: '{{ ''192.168.144.1/30'' | ansible.utils.ipsubnet(''192.168.144.0/24'') }}'
  - name: The fifth subnet /30 in a /24
    debug:
      msg: '{{ ''192.168.144.16/30'' | ansible.utils.ipsubnet(''192.168.144.0/24'') }}'


# PLAY [Ipsubnet filter plugin with different queries.] ****************************************************************
# TASK [convert IP address to subnet] *************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:10
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "192.168.144.5/32"
# }
#
# TASK [check if a given string is a subnet] ******************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:15
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "192.168.0.0/16"
# }
#
# TASK [Get the number of subnets a given subnet can be split into.] ******************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:20
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "16"
# }
#
# TASK [Get a 1st subnet] *************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:25
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "192.168.0.0/20"
# }
#
# TASK [Get a last subnet] ************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:30
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "192.168.240.0/20"
# }
#
# TASK [Get biggest subnet that contains that given IP address.] **********************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:35
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "192.168.144.0/20"
# }
#
# TASK [Get 1st smaller subnet by specifying 0 as index number] ***********************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:40
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "192.168.128.0/18"
# }
#
# TASK [Get last subnet] **************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:45
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "192.168.144.4/31"
# }
#
# TASK [The rank of the IP in the subnet (the IP is the 36870nth /32 of the subnet)] **************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:50
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "36870"
# }
#
# TASK [The rank in the /24 that contain the address] *********************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:55
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "6"
# }
#
# TASK [An IP with the subnet in the first /30 in a /24] ******************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:60
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "1"
# }
#
# TASK [he fifth subnet /30 in a /24] *************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_ipsubnet.yaml:65
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "5"
# }
"""

RETURN = """
  data:
    type: raw
    description:
      - Returns values valid for a particular query.
"""


@pass_environment
def _ipsubnet(*args, **kwargs):
    """Manipulate IPv4/IPv6 subnets"""
    keys = ["value", "query", "index"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="ipsubnet")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return ipsubnet(**updated_data)


def ipsubnet(value, query="", index=None):
    """Manipulate IPv4/IPv6 subnets"""

    vtype = ipaddr(value, "type")
    if not vtype:
        return False
    elif vtype == "address":
        v = ipaddr(value, "cidr")
    elif vtype == "network":
        v = ipaddr(value, "subnet")
    value = netaddr.IPNetwork(v).cidr
    vtype = ipaddr(value, "type")

    if not query:
        return to_text(value)

    vtotalbits = 128 if value.version == 6 else 32
    if query.isdigit():
        query = int(query)
        if query < 0 or query > vtotalbits:
            return False

        if index is None:
            if vtype == "address":
                return to_text(value.supernet(query)[0])
            elif vtype == "network":
                if query - value.prefixlen < 0:
                    msg = "Requested subnet size of {0} is invalid".format(
                        to_text(query),
                    )
                    raise AnsibleFilterError(msg)
                return to_text(2 ** (query - value.prefixlen))

        index = int(index)
        if vtype == "address":
            if index > vtotalbits + 1 - index or index < query - vtotalbits:
                return False
            return to_text(value.supernet(query)[index])
        elif vtype == "network":
            subnets = 2 ** (query - value.prefixlen)
            if index < 0:
                index = index + subnets
            if index < 0 or index >= subnets:
                # subnet index out of range
                return False
            return to_text(
                netaddr.IPNetwork(
                    to_text(
                        netaddr.IPAddress(
                            value.network.value + (index << vtotalbits - query),
                        ),
                    )
                    + "/"
                    + to_text(query),
                ),
            )
    else:
        vtype = ipaddr(query, "type")
        if vtype == "address":
            v = ipaddr(query, "cidr")
        elif vtype == "network":
            v = ipaddr(query, "subnet")
        else:
            msg = "You must pass a valid subnet or IP address; {0} is invalid".format(
                to_text(query),
            )
            raise AnsibleFilterError(msg)
        query = netaddr.IPNetwork(v)
        if (
            value.value >> vtotalbits - query.prefixlen
            == query.value >> vtotalbits - query.prefixlen
        ):
            return to_text(
                (
                    (
                        value.value
                        & 2 ** (value.prefixlen - query.prefixlen) - 1
                        << (vtotalbits - value.prefixlen)
                    )
                    >> (vtotalbits - value.prefixlen)
                )
                + 1,
            )
        msg = "{0} is not in the subnet {1}".format(value.cidr, query.cidr)
        raise AnsibleFilterError(msg)

    return False


class FilterModule(object):
    """IP address and network manipulation filters"""

    filter_map = {
        # IP addresses and networks
        "ipsubnet": _ipsubnet,
    }

    def filters(self):
        """ipsubnet filter"""
        if HAS_NETADDR:
            return self.filter_map
        else:
            return dict((f, partial(_need_netaddr, f)) for f in self.filter_map)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Samita Bhattacharjee (@samiib) <samitab@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: ndo_set_rule_policy
version_added: "2.12.0"
short_description: Manage Tenant Set Rule Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Tenant Set Rule Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v4.1 (NDO v5.1) and later.
author:
- Samita Bhattacharjee (@samiib)
options:
  template:
    description:
    - The name of the template.
    - The template must be a tenant template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the tenant template.
    - This parameter or O(template) is required.
    type: str
  name:
    description:
    - The name of the Set Rule Policy.
    type: str
    aliases: [ set_rule_policy ]
  uuid:
    description:
    - The UUID of the Set Rule Policy.
    - This parameter is required when the O(name) attribute needs to be updated.
    type: str
    aliases: [ set_rule_policy_uuid ]
  description:
    description:
    - The description of the Set Rule Policy.
    type: str
  set_community:
    description:
    - The community rule to add to the Set Rule Policy.
    - Providing an empty dictionary will remove the O(set_community={}) rule from the Set Rule Policy.
    type: dict
    suboptions:
      community:
        description:
        - The community value.
        type: str
      criteria:
        description:
        - The criteria to be applied to the community value.
        type: str
        choices: [ none, replace, append ]
  set_route_tag:
    description:
    - The route tag rule to add to the Set Rule Policy.
    - The value must be a number between 0 and 999999999.
    - Providing an empty string will remove the O(set_route_tag="") rule from the Set Rule Policy.
    type: str
  set_dampening:
    description:
    - The dampening rule to add to the Set Rule Policy.
    - Providing an empty dictionary will remove the O(set_dampening={}) rule from the Set Rule Policy.
    type: dict
    suboptions:
      half_life:
        description:
        - The half life time in minutes.
        - The value must be between 1 and 60.
        type: int
      reuse_limit:
        description:
        - The re-use limit.
        - The value must be between 1 and 20000.
        type: int
      suppress_limit:
        description:
        - The suppress limit.
        - The value must be between 1 and 20000.
        type: int
      max_suppress_time:
        description:
        - The maximum suppress time in minutes.
        - The value must be between 1 and 255.
        type: int
  set_weight:
    description:
    - The weight rule to add to the Set Rule Policy.
    - The value must be a number between 0 and 9999.
    - Providing an empty string will remove the O(set_weight="") rule from the Set Rule Policy.
    type: str
  set_next_hop:
    description:
    - The next hop rule to add to the Set Rule Policy.
    - The value must be a valid IPv4 or IPv6 address.
    - Providing an empty string will remove the O(set_next_hop="") rule from the Set Rule Policy.
    type: str
  set_preference:
    description:
    - The preference rule to add to the Set Rule Policy.
    - The value must be between 0 and 999999999.
    - Providing an empty string will remove the O(set_preference="") rule from the Set Rule Policy.
    type: str
  set_metric:
    description:
    - The metric rule to add to the Set Rule Policy.
    - The value must be between 0 and 999999999.
    - Providing an empty string will remove the O(set_metric="") rule from the Set Rule Policy.
    type: str
  set_metric_type:
    description:
    - The type of metric rule to add to the Set Rule Policy.
    - Providing an empty string will remove the O(set_metric_type="") from the Set Rule Policy.
    type: str
    choices: [ ospf_type1, ospf_type2, type1, type2, '' ]
  set_next_hop_propagate:
    description:
    - The next hop propagation value to add to the Set Rule Policy.
    - Providing a value of C(false) will remove next hop propagation from the Set Rule Policy.
    - O(set_next_hop_propagate) cannot be enabled with O(set_route_tag) configured.
    type: bool
  set_multi_path:
    description:
    - The multipath option to add to the Set Rule Policy.
    - Providing a value of C(false) will remove multipath from the Set Rule Policy.
    - Enabling multipath requires O(set_next_hop_propagate=true) to be enabled.
    type: bool
  set_additional_community:
    description:
    - The append list of communities to configure in the Set Rule Policy.
    - Providing an empty list will remove the O(set_additional_community=[]) from the Set Rule Policy.
    type: list
    elements: str
    aliases: [ set_add_communities ]
  set_as_path:
    description:
    - The Autonomous System (AS) path rules to add to the Set Rule Policy.
    - Providing an empty dictionary will remove the O(set_as_path={}) rules from the Set Rule Policy.
    type: dict
    suboptions:
      last_asn_count:
        description:
        - The prepend last AS count.
        - The value must be between 1 and 10.
        type: int
      path_asn_list:
        description:
        - The list of path AS numbers to prepend.
        - Providing an empty list will remove the O(set_as_path.path_asn_list=[]).
        type: list
        elements: dict
        suboptions:
          asn:
            description:
            - The AS number.
            - The value must be between 1 and 10.
            type: int
            required: true
          order:
            description:
            - The order of the AS number.
            - The value must be between 0 and 31.
            type: int
            required: true
  state:
    description:
    - Determines the desired state of the resource.
    - Use C(absent) to remove the resource.
    - Use C(query) to list the resource.
    - Use C(present) to create or update the resource.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant template.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new Set Rule Policy
  cisco.mso.ndo_set_rule_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: set_rule_policy_1
    description: Ansible Set Rule Policy
    set_community:
      community: no-export
      criteria: replace
    set_additional_community:
      - no-advertise
      - extended:color:35
    set_route_tag: 500
    set_dampening:
      half_life: 15
      reuse_limit: 750
      suppress_limit: 2000
      max_suppress_time: 60
    set_weight: 50
    set_next_hop: 'aa::'
    set_preference: 200
    set_metric: 100
    set_metric_type: type2
    set_as_path:
      last_asn_count: 10
      path_asn_list:
        - order: 11
          asn: 1
        - order: 10
          asn: 2
    state: present
  register: create_set_rule_policy_1

- name: Update the name of the Set Rule Policy using UUID
  cisco.mso.ndo_set_rule_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_id: "{{ create_tenant.current.templateId }}"
    uuid: "{{ create_set_rule_policy_1.current.uuid }}"
    name: set_rule_policy_1_updated
    state: present
  register: update_set_rule_policy_1

- name: Query an existing Set Rule Policy using UUID
  cisco.mso.ndo_set_rule_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    uuid: "{{ create_set_rule_policy_1.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query an existing Set Rule Policy using name
  cisco.mso.ndo_set_rule_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: set_rule_policy_1_updated
    state: query
  register: query_with_name

- name: Query all Set Rule Policies
  cisco.mso.ndo_set_rule_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    state: query
  register: query_all

- name: Remove set rules from a Set Rule Policy
  cisco.mso.ndo_set_rule_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: set_rule_policy_1_updated
    set_additional_community: []
    set_community: {}
    set_route_tag: ''
    set_propagate_next_hop: false
    state: present

- name: Delete an existing Set Rule Policy using UUID
  cisco.mso.ndo_set_rule_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    uuid: "{{ create_set_rule_policy_1.current.uuid }}"
    state: absent

- name: Delete an existing Set Rule Policy using Name
  cisco.mso.ndo_set_rule_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: set_rule_policy_1_updated
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data, check_if_all_elements_are_none
from ansible_collections.cisco.mso.plugins.module_utils.constants import ROUTE_MAP_METRIC_TYPE_MAP
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        name=dict(type="str", aliases=["set_rule_policy"]),
        uuid=dict(type="str", aliases=["set_rule_policy_uuid"]),
        description=dict(type="str"),
        set_community=dict(
            type="dict",
            options=dict(
                community=dict(type="str"),
                criteria=dict(type="str", choices=["none", "replace", "append"]),
            ),
        ),
        set_route_tag=dict(type="str"),
        set_weight=dict(type="str"),
        set_dampening=dict(
            type="dict",
            options=dict(
                half_life=dict(type="int"),
                reuse_limit=dict(type="int"),
                suppress_limit=dict(type="int"),
                max_suppress_time=dict(type="int"),
            ),
        ),
        set_next_hop=dict(type="str"),
        set_preference=dict(type="str"),
        set_metric=dict(type="str"),
        set_metric_type=dict(type="str", choices=list(ROUTE_MAP_METRIC_TYPE_MAP)),
        set_next_hop_propagate=dict(type="bool"),
        set_multi_path=dict(type="bool"),
        set_additional_community=dict(type="list", elements="str", aliases=["set_add_communities"]),
        set_as_path=dict(
            type="dict",
            options=dict(
                last_asn_count=dict(type="int"),
                path_asn_list=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        asn=dict(type="int", required=True),
                        order=dict(type="int", required=True),
                    ),
                ),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("template", "template_id"),
        ],
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
        required_one_of=[["template", "template_id"]],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    template_id = module.params.get("template_id")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    set_community = module.params.get("set_community")
    set_route_tag = get_int_empty_or_none(module.params.get("set_route_tag"))
    set_weight = get_int_empty_or_none(module.params.get("set_weight"))
    set_preference = get_int_empty_or_none(module.params.get("set_preference"))
    set_metric = get_int_empty_or_none(module.params.get("set_metric"))
    set_dampening = module.params.get("set_dampening")
    set_next_hop = module.params.get("set_next_hop")
    set_metric_type = ROUTE_MAP_METRIC_TYPE_MAP.get(module.params.get("set_metric_type"), None)
    set_next_hop_propagate = module.params.get("set_next_hop_propagate")
    set_multi_path = module.params.get("set_multi_path")
    set_additional_community = module.params.get("set_additional_community")
    if set_additional_community is not None and len(set_additional_community) > 0:
        set_additional_community = [dict(criteria="append", community=comm) for comm in set_additional_community]
    set_as_path = None
    if module.params.get("set_as_path") is not None:
        set_as_path = []
        last_asn_count = module.params.get("set_as_path", {}).get("last_asn_count")
        path_asn_list = module.params.get("set_as_path", {}).get("path_asn_list")
        if last_asn_count is not None:
            set_as_path.append(dict(criteria="prepend-last-as", asnCount=last_asn_count))
        if path_asn_list is not None:
            set_as_path.append(dict(criteria="prepend", pathASNs=path_asn_list))
    state = module.params.get("state")

    if set_dampening is not None and check_if_all_elements_are_none(set_dampening.values()):
        set_dampening = {}

    if set_community is not None and check_if_all_elements_are_none(set_community.values()):
        set_community = {}

    ops = []
    match = None

    mso_template = MSOTemplate(mso, "tenant", template, template_id)
    mso_template.validate_template("tenantPolicy")

    object_description = "Set Rule Policy"
    path = "/tenantPolicyTemplate/template/setRulePolicies"
    set_rule_policy_path = None

    match = mso_template.get_set_rule_policy_object(uuid, name, search_object=None, fail_module=False)
    if (name or uuid) and match:
        set_rule_policy_path = "{0}/{1}".format(path, match.index)
        mso_template.update_config_with_template_and_references(match.details)
        mso.existing = copy.deepcopy(match.details)
        mso.previous = copy.deepcopy(match.details)
    elif match:
        mso.existing = mso.previous = [mso_template.update_config_with_template_and_references(obj) for obj in match]

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

        if match:
            mso_values = dict(name=name, description=description)
            mso_values_remove = list()

            if set_route_tag == "" and match.details.get("setRouteTag"):
                mso_values_remove.append("setRouteTag")
            elif set_route_tag:
                mso_values["setRouteTag"] = set_route_tag

            if set_weight == "" and match.details.get("setWeight"):
                mso_values_remove.append("setWeight")
            elif set_weight:
                mso_values["setWeight"] = set_weight

            if set_next_hop == "" and match.details.get("setNextHop"):
                mso_values_remove.append("setNextHop")
            elif set_next_hop:
                mso_values["setNextHop"] = set_next_hop

            if set_preference == "" and match.details.get("setPreference"):
                mso_values_remove.append("setPreference")
            elif set_preference:
                mso_values["setPreference"] = set_preference

            if set_metric == "" and match.details.get("setMetric"):
                mso_values_remove.append("setMetric")
            elif set_metric:
                mso_values["setMetric"] = set_metric

            if set_metric_type == "" and match.details.get("setMetricType"):
                mso_values_remove.append("setMetricType")
            elif set_metric_type:
                mso_values["setMetricType"] = set_metric_type

            if set_next_hop_propagate is False and match.details.get("setNextHopPropagate"):
                mso_values_remove.append("setNextHopPropagate")
            elif set_next_hop_propagate:
                mso_values["setNextHopPropagate"] = set_next_hop_propagate

            if set_multi_path is False and match.details.get("setMultiPath"):
                mso_values_remove.append("setMultiPath")
            elif set_multi_path:
                mso_values["setMultiPath"] = set_multi_path

            if set_additional_community == [] and match.details.get("setAddCommunities"):
                mso_values_remove.append("setAddCommunities")
            elif set_additional_community:
                mso_values["setAddCommunities"] = set_additional_community

            if set_as_path == [] and match.details.get("setAsPath"):
                mso_values_remove.append("setAsPath")
            elif set_as_path:
                mso_values["setAsPath"] = set_as_path

            if set_community == {} and match.details.get("setCommunity"):
                mso_values_remove.append("setCommunity")
            elif set_community:
                mso_values[("setCommunity", "community")] = set_community.get("community")
                mso_values[("setCommunity", "criteria")] = set_community.get("criteria")

            if set_dampening == {} and match.details.get("setDampening"):
                mso_values_remove.append("setDampening")
            elif set_dampening:
                mso_values[("setDampening", "halfLife")] = set_dampening.get("half_life")
                mso_values[("setDampening", "reuseLimit")] = set_dampening.get("reuse_limit")
                mso_values[("setDampening", "suppressLimit")] = set_dampening.get("suppress_limit")
                mso_values[("setDampening", "maxSuppressTime")] = set_dampening.get("max_suppress_time")
            proposed_payload = copy.deepcopy(match.details)
            append_update_ops_data(ops, proposed_payload, set_rule_policy_path, mso_values, mso_values_remove)
            mso.sanitize(proposed_payload, collate=True)
        else:
            mso_values = dict(
                name=name,
                description=description,
                setRouteTag=set_route_tag,
                setWeight=set_weight,
                setNextHop=set_next_hop,
                setPreference=set_preference,
                setMetric=set_metric,
                setMetricType=set_metric_type,
                setNextHopPropagate=set_next_hop_propagate,
                setMultiPath=set_multi_path,
                setAddCommunities=set_additional_community,
                setAsPath=set_as_path,
                setCommunity=set_community,
                setDampening=(
                    dict(
                        halfLife=set_dampening.get("half_life"),
                        reuseLimit=set_dampening.get("reuse_limit"),
                        suppressLimit=set_dampening.get("suppress_limit"),
                        maxSuppressTime=set_dampening.get("max_suppress_time"),
                    )
                    if set_dampening
                    else None
                ),
            )
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path="{0}/-".format(path), value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=set_rule_policy_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = mso_template.get_set_rule_policy_object(uuid, name, search_object=response, fail_module=False)
        if match:
            mso_template.update_config_with_template_and_references(match.details)
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso_template.update_config_with_template_and_references(mso.proposed)
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


def get_int_empty_or_none(value):
    if value == "":
        return ""
    return int(value) if value is not None else None


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_vpc
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages global VPC configuration
description:
- Manages global VPC configuration
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
- Gabriele Gerbino (@GGabriele)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- The feature vpc must be enabled before this module can be used
- If not using management vrf, vrf must be globally on the device before using in
  the pkl config
- Although source IP isn't required on the command line it is required when using
  this module.  The PKL VRF must also be configured prior to using this module.
- Both pkl_src and pkl_dest are needed when changing PKL VRF.
options:
  domain:
    description:
    - VPC domain
    required: true
    type: str
  role_priority:
    description:
    - Role priority for device. Remember lower is better.
    type: str
  system_priority:
    description:
    - System priority device.  Remember they must match between peers.
    type: str
  pkl_src:
    description:
    - Source IP address used for peer keepalive link
    type: str
  pkl_dest:
    description:
    - Destination (remote) IP address used for peer keepalive link
    - pkl_dest is required whenever pkl options are used.
    type: str
  pkl_vrf:
    description:
    - VRF used for peer keepalive link
    - The VRF must exist on the device before using pkl_vrf.
    - "(Note) 'default' is an overloaded term: Default vrf context for pkl_vrf is
      'management'; 'pkl_vrf: default' refers to the literal 'default' rib."
    type: str
  peer_gw:
    description:
    - Enables/Disables peer gateway
    type: bool
  peer_sw:
    description:
    - Enables/Disables peer-switch
    type: bool
  auto_recovery:
    description:
    - Enables/Disables auto recovery on platforms that support disable
    - timers are not modifiable with this attribute
    - mutually exclusive with auto_recovery_reload_delay
    type: bool
  auto_recovery_reload_delay:
    description:
    - Manages auto-recovery reload-delay timer in seconds
    - mutually exclusive with auto_recovery
    type: str
  delay_restore:
    description:
    - manages delay restore command and config value in seconds
    type: str
  delay_restore_interface_vlan:
    description:
    - manages delay restore interface-vlan command and config value in seconds
    - not supported on all platforms
    type: str
  delay_restore_orphan_port:
    description:
    - manages delay restore orphan-port command and config value in seconds
    - not supported on all platforms
    type: str
  state:
    description:
    - Manages desired state of the resource
    default: present
    choices:
    - present
    - absent
    type: str
"""

EXAMPLES = """
- name: configure a simple asn
  cisco.nxos.nxos_vpc:
    domain: 100
    role_priority: 1000
    system_priority: 2000
    pkl_dest: 192.168.100.4
    pkl_src: 10.1.100.20
    peer_gw: true
    peer_sw: true
    auto_recovery: true

- name: configure
  cisco.nxos.nxos_vpc:
    domain: 100
    role_priority: 32667
    system_priority: 2000
    peer_gw: true
    peer_sw: true
    pkl_src: 10.1.100.2
    pkl_dest: 192.168.100.4
    auto_recovery: true

- name: Configure VPC with delay restore and existing keepalive VRF
  cisco.nxos.nxos_vpc:
    domain: 10
    role_priority: 28672
    system_priority: 2000
    delay_restore: 180
    peer_gw: true
    peer_sw: true
    pkl_src: 1.1.1.2
    pkl_dest: 1.1.1.1
    pkl_vrf: vpckeepalive
    auto_recovery: true
"""

RETURN = """
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["vpc domain 100",
            "peer-keepalive destination 192.168.100.4 source 10.1.100.20 vrf management",
            "auto-recovery", "peer-gateway","peer-switch"]
"""

import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_config,
    load_config,
    run_commands,
)


CONFIG_ARGS = {
    "role_priority": "role priority {role_priority}",
    "system_priority": "system-priority {system_priority}",
    "delay_restore": "delay restore {delay_restore}",
    "delay_restore_interface_vlan": "delay restore interface-vlan {delay_restore_interface_vlan}",
    "delay_restore_orphan_port": "delay restore orphan-port {delay_restore_orphan_port}",
    "peer_sw": "{peer_sw} peer-switch",
    "peer_gw": "{peer_gw} peer-gateway",
    "auto_recovery": "{auto_recovery} auto-recovery",
    "auto_recovery_reload_delay": "auto-recovery reload-delay {auto_recovery_reload_delay}",
}

PARAM_TO_DEFAULT_KEYMAP = {
    "delay_restore": "60",
    "delay_restore_interface_vlan": "10",
    "delay_restore_orphan_port": "0",
    "role_priority": "32667",
    "system_priority": "32667",
    "peer_gw": False,
    "peer_sw": False,
    "auto_recovery_reload_delay": 240,
}


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_vrf_list(module):
    try:
        body = run_commands(module, ["show vrf all | json"])[0]
        vrf_table = body["TABLE_vrf"]["ROW_vrf"]
    except (KeyError, AttributeError):
        return []

    vrf_list = []
    if vrf_table:
        for each in vrf_table:
            vrf_list.append(str(each["vrf_name"].lower()))

    return vrf_list


def get_auto_recovery_default(module):
    auto = False
    data = run_commands(module, ["show inventory | json"])[0]
    pid = data["TABLE_inv"]["ROW_inv"][0]["productid"]
    if re.search(r"N7K", pid):
        auto = True
    elif re.search(r"N9K", pid):
        data = run_commands(module, ["show hardware | json"])[0]
        ver = data.get("kickstart_ver_str", "")

        if re.search(r"7.0\(3\)F", ver):
            auto = True

    return auto


def get_vpc(module):
    body = run_commands(module, ["show vpc | json"])[0]
    if body:
        domain = str(body["vpc-domain-id"])
    else:
        body = run_commands(module, ["show run vpc | inc domain"])[0]
        if body:
            domain = body.split()[2]
        else:
            domain = "not configured"

    vpc = {}
    if domain != "not configured":
        run = get_config(module, flags=["vpc all"])
        if run:
            vpc["domain"] = domain
            for key in PARAM_TO_DEFAULT_KEYMAP.keys():
                vpc[key] = PARAM_TO_DEFAULT_KEYMAP.get(key)
            vpc["auto_recovery"] = get_auto_recovery_default(module)
            vpc_list = run.split("\n")
            for each in vpc_list:
                if "role priority" in each:
                    line = each.split()
                    vpc["role_priority"] = line[-1]
                if "system-priority" in each:
                    line = each.split()
                    vpc["system_priority"] = line[-1]
                if re.search(r"delay restore \d+", each):
                    line = each.split()
                    vpc["delay_restore"] = line[-1]
                if "delay restore interface-vlan" in each:
                    line = each.split()
                    vpc["delay_restore_interface_vlan"] = line[-1]
                if "delay restore orphan-port" in each:
                    line = each.split()
                    vpc["delay_restore_orphan_port"] = line[-1]
                if "auto-recovery" in each:
                    vpc["auto_recovery"] = False if "no " in each else True
                    line = each.split()
                    vpc["auto_recovery_reload_delay"] = line[-1]
                if "peer-gateway" in each:
                    vpc["peer_gw"] = False if "no " in each else True
                if "peer-switch" in each:
                    vpc["peer_sw"] = True
                if "peer-keepalive destination" in each:
                    # destination is reqd; src & vrf are optional
                    m = re.search(
                        r"destination (?P<pkl_dest>[\d.]+)"
                        r"(?:.* source (?P<pkl_src>[\d.]+))*"
                        r"(?:.* vrf (?P<pkl_vrf>\S+))*",
                        each,
                    )
                    if m:
                        for pkl in m.groupdict().keys():
                            if m.group(pkl):
                                vpc[pkl] = m.group(pkl)
    return vpc


def pkl_dependencies(module, delta, existing):
    """peer-keepalive dependency checking.
    1. 'destination' is required with all pkl configs.
    2. If delta has optional pkl keywords present, then all optional pkl
       keywords in existing must be added to delta, otherwise the device cli
       will remove those values when the new config string is issued.
    3. The desired behavior for this set of properties is to merge changes;
       therefore if an optional pkl property exists on the device but not
       in the playbook, then that existing property should be retained.
    Example:
      CLI:       peer-keepalive dest 10.1.1.1 source 10.1.1.2 vrf orange
      Playbook:  {pkl_dest: 10.1.1.1, pkl_vrf: blue}
      Result:    peer-keepalive dest 10.1.1.1 source 10.1.1.2 vrf blue
    """
    pkl_existing = [i for i in existing.keys() if i.startswith("pkl")]
    for pkl in pkl_existing:
        param = module.params.get(pkl)
        if not delta.get(pkl):
            if param and param == existing[pkl]:
                # delta is missing this param because it's idempotent;
                # however another pkl command has changed; therefore
                # explicitly add it to delta so that the cli retains it.
                delta[pkl] = existing[pkl]
            elif param is None and existing[pkl]:
                # retain existing pkl commands even if not in playbook
                delta[pkl] = existing[pkl]


def get_commands_to_config_vpc(module, vpc, domain, existing):
    vpc = dict(vpc)

    domain_only = vpc.get("domain")

    commands = []
    if "pkl_dest" in vpc:
        pkl_command = "peer-keepalive destination {pkl_dest}".format(**vpc)
        if "pkl_src" in vpc:
            pkl_command += " source {pkl_src}".format(**vpc)
        if "pkl_vrf" in vpc:
            pkl_command += " vrf {pkl_vrf}".format(**vpc)
        commands.append(pkl_command)

    if "auto_recovery" in vpc:
        if not vpc.get("auto_recovery"):
            vpc["auto_recovery"] = "no"
        else:
            vpc["auto_recovery"] = ""

    if "peer_gw" in vpc:
        if not vpc.get("peer_gw"):
            vpc["peer_gw"] = "no"
        else:
            vpc["peer_gw"] = ""

    if "peer_sw" in vpc:
        if not vpc.get("peer_sw"):
            vpc["peer_sw"] = "no"
        else:
            vpc["peer_sw"] = ""
    for param in vpc:
        command = CONFIG_ARGS.get(param)
        if command is not None:
            command = command.format(**vpc).strip()
            if "peer-gateway" in command:
                commands.append("terminal dont-ask")
            if "peer-switch" in command:
                commands.append("terminal dont-ask")
            commands.append(command)

    if commands or domain_only:
        commands.insert(0, "vpc domain {0}".format(domain))
    return commands


def main():
    argument_spec = dict(
        domain=dict(required=True, type="str"),
        role_priority=dict(required=False, type="str"),
        system_priority=dict(required=False, type="str"),
        pkl_src=dict(required=False),
        pkl_dest=dict(required=False),
        pkl_vrf=dict(required=False),
        peer_gw=dict(required=False, type="bool"),
        peer_sw=dict(required=False, type="bool"),
        auto_recovery=dict(required=False, type="bool"),
        auto_recovery_reload_delay=dict(required=False, type="str"),
        delay_restore=dict(required=False, type="str"),
        delay_restore_interface_vlan=dict(required=False, type="str"),
        delay_restore_orphan_port=dict(required=False, type="str"),
        state=dict(choices=["absent", "present"], default="present"),
    )

    mutually_exclusive = [("auto_recovery", "auto_recovery_reload_delay")]
    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    warnings = list()
    results = {"changed": False, "warnings": warnings}

    domain = module.params["domain"]
    role_priority = module.params["role_priority"]
    system_priority = module.params["system_priority"]
    pkl_src = module.params["pkl_src"]
    pkl_dest = module.params["pkl_dest"]
    pkl_vrf = module.params["pkl_vrf"]
    peer_gw = module.params["peer_gw"]
    peer_sw = module.params["peer_sw"]
    auto_recovery = module.params["auto_recovery"]
    auto_recovery_reload_delay = module.params["auto_recovery_reload_delay"]
    delay_restore = module.params["delay_restore"]
    delay_restore_interface_vlan = module.params["delay_restore_interface_vlan"]
    delay_restore_orphan_port = module.params["delay_restore_orphan_port"]
    state = module.params["state"]

    args = dict(
        domain=domain,
        role_priority=role_priority,
        system_priority=system_priority,
        pkl_src=pkl_src,
        pkl_dest=pkl_dest,
        pkl_vrf=pkl_vrf,
        peer_gw=peer_gw,
        peer_sw=peer_sw,
        auto_recovery=auto_recovery,
        auto_recovery_reload_delay=auto_recovery_reload_delay,
        delay_restore=delay_restore,
        delay_restore_interface_vlan=delay_restore_interface_vlan,
        delay_restore_orphan_port=delay_restore_orphan_port,
    )

    if not pkl_dest:
        if pkl_src:
            module.fail_json(msg="dest IP for peer-keepalive is required" " when src IP is present")
        elif pkl_vrf:
            if pkl_vrf != "management":
                module.fail_json(
                    msg="dest and src IP for peer-keepalive are required" " when vrf is present",
                )
            else:
                module.fail_json(
                    msg="dest IP for peer-keepalive is required" " when vrf is present",
                )
    if pkl_vrf:
        if pkl_vrf.lower() not in get_vrf_list(module):
            module.fail_json(
                msg="The VRF you are trying to use for the peer "
                "keepalive link is not on device yet. Add it"
                " first, please.",
            )
    proposed = dict((k, v) for k, v in args.items() if v is not None)
    existing = get_vpc(module)

    commands = []
    if state == "present":
        delta = {}
        for key, value in proposed.items():
            if str(value).lower() == "default" and key != "pkl_vrf":
                # 'default' is a reserved word for vrf
                value = PARAM_TO_DEFAULT_KEYMAP.get(key)
            if existing.get(key) != value:
                delta[key] = value

        if delta:
            pkl_dependencies(module, delta, existing)
            command = get_commands_to_config_vpc(module, delta, domain, existing)
            commands.append(command)
    elif state == "absent":
        if existing:
            if domain != existing["domain"]:
                module.fail_json(
                    msg="You are trying to remove a domain that " "does not exist on the device",
                )
            else:
                commands.append("terminal dont-ask")
                commands.append("no vpc domain {0}".format(domain))

    cmds = flatten_list(commands)
    results["commands"] = cmds

    if cmds:
        results["changed"] = True
        if not module.check_mode:
            load_config(module, cmds)
            if "configure" in cmds:
                cmds.pop(0)

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2024, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: django_command
author:
  - Alexei Znamensky (@russoz)
short_description: Run Django admin commands
version_added: 9.0.0
description:
  - This module allows the execution of arbitrary Django admin commands.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._django
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  command:
    description:
      - Django admin command. It must be a valid command accepted by C(python -m django) at the target system.
    type: str
    required: true
  extra_args:
    type: list
    elements: str
    description:
      - List of extra arguments passed to the django admin command.
"""

EXAMPLES = r"""
- name: Check the project
  community.general.django_command:
    command: check
    settings: myproject.settings

- name: Check the project in specified python path, using virtual environment
  community.general.django_command:
    command: check
    settings: fancysite.settings
    pythonpath: /home/joedoe/project/fancysite
    venv: /home/joedoe/project/fancysite/venv
"""

RETURN = r"""
run_info:
  description: Command-line execution information.
  type: dict
  returned: success and O(verbosity) >= 3
version:
  description: Version of Django.
  type: str
  returned: always
  sample: 5.1.2
  version_added: 10.0.0
"""

import shlex

from ansible_collections.community.general.plugins.module_utils._cmd_runner import cmd_runner_fmt
from ansible_collections.community.general.plugins.module_utils._django import DjangoModuleHelper


class DjangoCommand(DjangoModuleHelper):
    module = dict(
        argument_spec=dict(
            command=dict(type="str", required=True),
            extra_args=dict(type="list", elements="str"),
        ),
        supports_check_mode=False,
    )
    arg_formats = dict(
        extra_args=cmd_runner_fmt.as_list(),
    )
    django_admin_arg_order = "extra_args"

    def __init_module__(self):
        self.vars.command = shlex.split(self.vars.command)


def main():
    DjangoCommand.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: ucs_storage_profile
short_description: Configures storage profiles on Cisco UCS Manager
description:
- Configures storage profiles on Cisco UCS Manager.
extends_documentation_fragment: cisco.ucs.ucs
options:
  state:
    description:
    - If C(present), will verify that the storage profile is present and will create if needed.
    - If C(absent), will verify that the storage profile is absent and will delete if needed.
    choices: [ absent, present ]
    default: present
    type: str
  name:
    description:
    - The name of the storage profile.
    - This name can be between 1 and 16 alphanumeric characters.
    - "You cannot use spaces or any special characters other than - (hyphen), \"_\" (underscore), : (colon), and . (period)."
    - You cannot change this name after profile is created.
    required: yes
    type: str
  description:
    description:
    - The user-defined description of the storage profile.
    - Enter up to 256 characters.
    - "You can use any characters or spaces except the following:"
    - "` (accent mark), \ (backslash), ^ (carat), \" (double quote), = (equal sign), > (greater than), < (less than), or ' (single quote)."
    aliases: [ descr ]
    type: str
  local_luns:
    description:
    - List of Local LUNs used by the storage profile.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The name of the local LUN.
        required: yes
        type: str
      size:
        description:
        - Size of this LUN in GB.
        - The size can range from 1 to 10240 GB.
        default: '1'
        type: str
      auto_deploy:
        description:
        - Whether the local LUN should be automatically deployed or not.
        choices: [ auto-deploy, no-auto-deploy ]
        default: auto-deploy
        type: str
      expand_to_avail:
        description:
        - Specifies that this LUN can be expanded to use the entire available disk group.
        - For each service profile, only one LUN can use this option.
        - Expand To Available option is not supported for already deployed LUN.
        choices: ['no', 'yes']
        default: 'no'
        type: str
      fractional_size:
        description:
        - Fractional size of this LUN in MB.
        default: '0'
        type: str
      disk_policy_name:
        description:
        - The disk group configuration policy to be applied to this local LUN.
        type: str
      state:
        description:
        - If C(present), will verify local LUN is present on profile.
          If C(absent), will verify local LUN is absent on profile.
        choices: [ absent, present ]
        default: present
        type: str
  org_dn:
    description:
    - The distinguished name (dn) of the organization where the resource is assigned.
    default: org-root
    type: str
requirements:
- ucsmsdk
author:
- Sindhu Sudhir (@sisudhir)
- David Soper (@dsoper2)
- John McDonough (@movinalot)
- CiscoUcs (@CiscoUcs)
'''

EXAMPLES = r'''
- name: Configure Storage Profile
  cisco.ucs.ucs_storage_profile:
    hostname: 172.16.143.150
    username: admin
    password: password
    name: DEE-StgProf
    local_luns:
    - name: Boot-LUN
      size: '60'
      disk_policy_name: DEE-DG
    - name: Data-LUN
      size: '200'
      disk_policy_name: DEE-DG

- name: Remove Storage Profile
  cisco.ucs.ucs_storage_profile:
    hostname: 172.16.143.150
    username: admin
    password: password
    name: DEE-StgProf
    state: absent

- name: Remove Local LUN from Storage Profile
  cisco.ucs.ucs_storage_profile:
    hostname: 172.16.143.150
    username: admin
    password: password
    name: DEE-StgProf
    local_luns:
    - name: Data-LUN
      state: absent
'''

RETURN = r'''
#
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.ucs.plugins.module_utils.ucs import UCSModule, ucs_argument_spec


def main():
    local_lun = dict(
        name=dict(type='str', required=True),
        state=dict(type='str', default='present', choices=['present', 'absent']),
        size=dict(type='str', default='1'),
        auto_deploy=dict(type='str', default='auto-deploy', choices=['auto-deploy', 'no-auto-deploy']),
        expand_to_avail=dict(type='str', default='no', choices=['no', 'yes']),
        fractional_size=dict(type='str', default='0'),
        disk_policy_name=dict(type='str'),
    )
    argument_spec = ucs_argument_spec.copy()
    argument_spec.update(
        org_dn=dict(type='str', default='org-root'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        local_luns=dict(type='list', elements='dict', options=local_lun),
        state=dict(type='str', default='present', choices=['present', 'absent']),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    ucs = UCSModule(module)

    err = False

    # UCSModule creation above verifies ucsmsdk is present and exits on failure.  Additional imports are done below.
    from ucsmsdk.mometa.lstorage.LstorageProfile import LstorageProfile
    from ucsmsdk.mometa.lstorage.LstorageDasScsiLun import LstorageDasScsiLun

    ucs.result['changed'] = False
    try:
        mo_exists = False
        props_match = False
        # dn is <org_dn>/profile-<name>
        dn = module.params['org_dn'] + '/profile-' + module.params['name']

        mo = ucs.login_handle.query_dn(dn)
        if mo:
            mo_exists = True

        if module.params['state'] == 'absent':
            # mo must exist but all properties do not have to match
            if mo_exists:
                if not module.check_mode:
                    ucs.login_handle.remove_mo(mo)
                    ucs.login_handle.commit()
                ucs.result['changed'] = True
        else:
            if mo_exists:
                # check top-level mo props
                kwargs = dict(descr=module.params['description'])
                if mo.check_prop_match(**kwargs):
                    # top-level props match, check next level mo/props
                    if not module.params.get('local_luns'):
                        props_match = True
                    else:
                        # check local lun props
                        for lun in module.params['local_luns']:
                            child_dn = dn + '/das-scsi-lun-' + lun['name']
                            mo_1 = ucs.login_handle.query_dn(child_dn)
                            if lun['state'] == 'absent':
                                if mo_1:
                                    props_match = False
                                    break
                            else:
                                if mo_1:
                                    kwargs = dict(size=str(lun['size']))
                                    kwargs['auto_deploy'] = lun['auto_deploy']
                                    kwargs['expand_to_avail'] = lun['expand_to_avail']
                                    kwargs['fractional_size'] = str(lun['fractional_size'])
                                    kwargs['local_disk_policy_name'] = lun['disk_policy_name']
                                    if mo_1.check_prop_match(**kwargs):
                                        props_match = True
                                else:
                                    props_match = False
                                    break

            if not props_match:
                if not module.check_mode:
                    # create if mo does not already exist
                    mo = LstorageProfile(
                        parent_mo_or_dn=module.params['org_dn'],
                        name=module.params['name'],
                        descr=module.params['description'],
                    )

                    if module.params.get('local_luns'):
                        for lun in module.params['local_luns']:
                            if lun['state'] == 'absent':
                                child_dn = dn + '/das-scsi-lun-' + lun['name']
                                mo_1 = ucs.login_handle.query_dn(child_dn)
                                ucs.login_handle.remove_mo(mo_1)
                            else:
                                mo_1 = LstorageDasScsiLun(
                                    parent_mo_or_dn=mo,
                                    name=lun['name'],
                                    size=str(lun['size']),
                                    auto_deploy=lun['auto_deploy'],
                                    expand_to_avail=lun['expand_to_avail'],
                                    fractional_size=str(lun['fractional_size']),
                                    local_disk_policy_name=lun['disk_policy_name'],
                                )

                    ucs.login_handle.add_mo(mo, True)
                    ucs.login_handle.commit()
                ucs.result['changed'] = True

    except Exception as e:
        err = True
        ucs.result['msg'] = "setup error: %s " % str(e)

    if err:
        module.fail_json(**ucs.result)
    module.exit_json(**ucs.result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos bgp_neighbor_address_family fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.bgp_neighbor_address_family.bgp_neighbor_address_family import (
    Bgp_neighbor_address_familyArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.bgp_neighbor_address_family import (
    Bgp_neighbor_address_familyTemplate,
)


class Bgp_neighbor_address_familyFacts(object):
    """The nxos bgp_neighbor_address_family facts class"""

    def __init__(self, module):
        self._module = module
        self.argument_spec = Bgp_neighbor_address_familyArgs.argument_spec

    def get_config(self, connection):
        """Wrapper method for `connection.get()`
        This method exists solely to allow the unit test framework to mock device connection calls.
        """
        return connection.get("show running-config | section '^router bgp'")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Bgp_neighbor_address_family network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = {}

        if not data:
            data = self.get_config(connection)

        data = self._flatten_config(data)

        # parse native config using the Bgp_neighbor_address_family template
        bgp_neighbor_address_family_parser = Bgp_neighbor_address_familyTemplate(lines=data)
        objs = bgp_neighbor_address_family_parser.parse()

        if objs:
            top_lvl_nbrs = objs.get("vrfs", {}).pop("vrf_", {})
            objs["neighbors"] = self._post_parse(top_lvl_nbrs).get("neighbors", [])

            if "vrfs" in objs:
                for vrf in objs["vrfs"].values():
                    vrf["neighbors"] = self._post_parse(vrf)["neighbors"]
                objs["vrfs"] = list(objs["vrfs"].values())

        ansible_facts["ansible_network_resources"].pop("bgp_neighbor_address_family", None)

        params = utils.remove_empties(utils.validate_config(self.argument_spec, {"config": objs}))

        facts["bgp_neighbor_address_family"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts

    def _post_parse(self, data):
        if "neighbors" in data:
            data["neighbors"] = sorted(
                list(data["neighbors"].values()),
                key=lambda k, s="neighbor_address": k[s],
            )
            for nbr in data["neighbors"]:
                nbr["address_family"] = sorted(
                    list(nbr["address_family"].values()),
                    key=lambda k: (k["afi"], k.get("safi", "")),
                )
        return data

    def _flatten_config(self, data):
        """Flatten contexts in the BGP
            running-config for easier parsing.
            Only neighbor AF contexts are returned.
        :param data: str
        :returns: flattened running config
        """
        data = data.split("\n")
        nbr_af_cxt = []
        context = ""
        cur_vrf = ""
        cur_nbr_indent = None
        in_nbr_cxt = False
        in_af = False

        # this is the "router bgp <asn>" line
        nbr_af_cxt.append(data[0])
        for x in data:
            cur_indent = len(x) - len(x.lstrip())
            x = x.strip()
            if x.startswith("vrf"):
                cur_vrf = x + " "
                in_nbr_cxt = False
            elif x.startswith("neighbor"):
                in_nbr_cxt = True
                in_af = False
                cur_nbr_indent = cur_indent
                context = x
                if cur_vrf:
                    context = cur_vrf + context
            elif in_nbr_cxt and cur_indent > cur_nbr_indent:
                if x.startswith("address-family"):
                    in_af = True
                    x = context + " " + x
                if in_af:
                    nbr_af_cxt.append(x)
            else:
                in_nbr_cxt = False

        return nbr_af_cxt



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.rulesetsoperations?view=azure-python

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_afdruleset
version_added: "3.4.0"
short_description: Manage an Azure Front Door Rule Set
description:
    - Create, update and delete an Azure Front Door Rule Set to be used by a Front Door Service Profile created using azure_rm_cdnprofile.

options:
    name:
        description:
            - Name of the Front Door Rule Set.
        required: true
        type: str
    profile_name:
        description:
            - Name of the Front Door Profile.
        required: true
        type: str
    resource_group:
        description:
            - Name of a resource group where the CDN front door ruleset exists or will be created.
        required: true
        type: str
    state:
        description:
            - Assert the state of the CDN profile. Use C(present) to create or update a CDN profile and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Create a Ruleset
  azure_rm_afdruleset:
    name: myRuleset
    profile_name: myProfile
    resource_group: myResourceGroup
    state: present

- name: Delete a Ruleset
  azure_rm_afdruleset:
    name: myRuleset
    profile_name: myProfile
    resource_group: myResourceGroup
    state: absent
'''
RETURN = '''
id:
    description: Resource ID
    returned: always
    type: str
    example: "/subscriptions/xxxxxx-xxxx-xxxx-xxxx-xxxxxxx/resourcegroups/myRG/providers/Microsoft.Cdn/profiles/myProf/rulesets/myRS"
'''
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMRuleSet(AzureRMModuleBase):
    """Class representing a Ruleset for Azure Frontdoor Standard or Premium"""

    def __init__(self):
        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True
            ),
            profile_name=dict(
                type='str',
                required=True
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent'],
                required=False
            )
        )

        self.name = None
        self.profile_name = None
        self.resource_group = None
        self.state = None

        self.results = dict(changed=False)

        super(AzureRMRuleSet, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_check_mode=True,
            supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        response = self.get_ruleset()

        self.results['changed'] = False
        self.results['id'] = None

        if self.state == 'present':

            if not response:
                self.log("Need to create the Rule Set")
                if not self.check_mode:
                    new_response = self.create_ruleset()
                    self.results['id'] = new_response['id']

                self.results['changed'] = True

            else:
                self.results['id'] = response['id']
                self.log('Results : {0}'.format(response))

        elif self.state == 'absent':
            if not response:
                self.log("Rule Set {0} does not exist.".format(self.name))
            else:
                self.log("Need to delete the Rule Set")
                self.results['changed'] = True

                self.results['id'] = response['id']
                if not self.check_mode:
                    self.delete_ruleset()

        return self.results

    def create_ruleset(self):
        '''
        Creates an Azure Rule Set.

        :return: deserialized Azure Rule Set instance state dictionary
        '''
        self.log("Creating the Azure Rule Set instance {0}".format(self.name))

        try:
            response = self.cdn_client.rule_sets.create(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                rule_set_name=self.name
            )
            return ruleset_to_dict(response)
        except AttributeError as exc:
            self.fail("Please ensure azure-mgmt-cdn is upgraded to version 13.1.0 or greater: {0}".format(str(exc)))
        except Exception as exc:
            self.log('Error attempting to create Azure Rule Set instance.')
            self.fail("Error Creating Azure Rule Set instance: {0}".format(str(exc)))

    def delete_ruleset(self):
        '''
        Deletes the specified Azure Rule Set in the specified subscription and resource group.

        :return: bool
        '''
        self.log("Deleting the Rule Set {0}".format(self.name))
        try:
            poller = self.cdn_client.rule_sets.begin_delete(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                rule_set_name=self.name)
            self.get_poller_result(poller)
            return True
        except Exception as exc:
            self.log('Error attempting to delete the Rule Set.')
            self.fail("Error deleting the Rule Set: {0}".format(str(exc)))
            return False

    def get_ruleset(self):
        '''
        Gets the properties of the specified Rule Set.

        :return: deserialized Rule Set state dictionary
        '''
        self.log(
            "Checking if the Rule Set {0} is present".format(self.name))
        try:
            response = self.cdn_client.rule_sets.get(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                rule_set_name=self.name,
            )
            self.log("Response : {0}".format(response))
            self.log("Rule Set : {0} found".format(response.name))
            return ruleset_to_dict(response)
        except Exception as err:
            self.log('Did not find the Rule Set.' + err.args[0])
            return False


def ruleset_to_dict(ruleset):
    '''
    Sets the properties of the specified Rule Set.

    :return: deserialized Rule Set state dictionary
    '''
    return dict(
        deployment_status=ruleset.deployment_status,
        id=ruleset.id,
        name=ruleset.name,
        provisioning_state=ruleset.provisioning_state,
        type=ruleset.type
    )


def main():
    """Main execution"""
    AzureRMRuleSet()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# This module is proudly sponsored by CGI (www.cgi.com) and
# KPN (www.kpn.com).
# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: icinga2_host
short_description: Manage a host in Icinga2
description:
  - Add or remove a host to Icinga2 through the API.
  - See U(https://www.icinga.com/docs/icinga2/latest/doc/12-icinga2-api/).
author: "Jurgen Brand (@t794104)"
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  url:
    type: str
    description:
      - HTTP, HTTPS, or FTP URL in the form V((http|https|ftp\)://[user[:pass]]@host.domain[:port]/path).
  use_proxy:
    description:
      - If V(false), it does not use a proxy, even if one is defined in an environment variable on the target hosts.
    type: bool
    default: true
  validate_certs:
    description:
      - If V(false), SSL certificates are not validated. This should only be used on personally controlled sites using self-signed
        certificates.
    type: bool
    default: true
  url_username:
    type: str
    description:
      - The username for use in HTTP basic authentication.
      - This parameter can be used without O(url_password) for sites that allow empty passwords.
  url_password:
    type: str
    description:
      - The password for use in HTTP basic authentication.
      - If the O(url_username) parameter is not specified, the O(url_password) parameter is not used.
  force_basic_auth:
    description:
      - C(httplib2), the library used by Ansible's HTTP request code only sends authentication information when a webservice
        responds to an initial request with a 401 status. Since some basic auth services do not properly send a 401, logins
        may fail. This option forces the sending of the Basic authentication header upon initial request.
    type: bool
    default: false
  client_cert:
    type: path
    description:
      - PEM formatted certificate chain file to be used for SSL client authentication. This file can also include the key
        as well, and if the key is included, O(client_key) is not required.
  client_key:
    type: path
    description:
      - PEM formatted file that contains your private key to be used for SSL client authentication. If O(client_cert) contains
        both the certificate and key, this option is not required.
  state:
    type: str
    description:
      - Apply feature state.
    choices: ["present", "absent"]
    default: present
  name:
    type: str
    description:
      - Name used to create / delete the host. This does not need to be the FQDN, but does needs to be unique.
    required: true
    aliases: [host]
  zone:
    type: str
    description:
      - The zone from where this host should be polled.
  template:
    type: str
    description:
      - The template used to define the host.
      - Template cannot be modified after object creation.
  check_command:
    type: str
    description:
      - The command used to check if the host is alive.
    default: "hostalive"
  display_name:
    type: str
    description:
      - The name used to display the host.
      - If not specified, it defaults to the value of the O(name) parameter.
  ip:
    type: str
    description:
      - The IP address of the host.
      - This is no longer required since community.general 8.0.0.
  variables:
    type: dict
    description:
      - Dictionary of variables.
extends_documentation_fragment:
  - ansible.builtin.url
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Add host to icinga
  community.general.icinga2_host:
    url: "https://icinga2.example.com"
    url_username: "ansible"
    url_password: "a_secret"
    state: present
    name: "{{ ansible_fqdn }}"
    ip: "{{ ansible_default_ipv4.address }}"
    variables:
      foo: "bar"
  delegate_to: 127.0.0.1
"""

RETURN = r"""
name:
  description: The name used to create, modify or delete the host.
  type: str
  returned: always
data:
  description: The data structure used for create, modify or delete of the host.
  type: dict
  returned: always
"""

import json

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url, url_argument_spec


# ===========================================
# Icinga2 API class
#
class icinga2_api:
    module = None

    def __init__(self, module):
        self.module = module

    def call_url(self, path, data="", method="GET"):
        headers = {
            "Accept": "application/json",
            "X-HTTP-Method-Override": method,
        }
        url = f"{self.module.params.get('url')}/{path}"
        rsp, info = fetch_url(
            module=self.module,
            url=url,
            data=data,
            headers=headers,
            method=method,
            use_proxy=self.module.params["use_proxy"],
        )
        body = ""
        if rsp:
            body = json.loads(rsp.read())
        if info["status"] >= 400:
            body = info["body"]
        return {"code": info["status"], "data": body}

    def check_connection(self):
        ret = self.call_url("v1/status")
        return ret["code"] == 200

    def exists(self, hostname):
        data = {
            "filter": f'match("{hostname}", host.name)',
        }
        ret = self.call_url(path="v1/objects/hosts", data=self.module.jsonify(data))
        if ret["code"] == 200:
            if len(ret["data"]["results"]) == 1:
                return True
        return False

    def create(self, hostname, data):
        ret = self.call_url(path=f"v1/objects/hosts/{hostname}", data=self.module.jsonify(data), method="PUT")
        return ret

    def delete(self, hostname):
        data = {"cascade": 1}
        ret = self.call_url(path=f"v1/objects/hosts/{hostname}", data=self.module.jsonify(data), method="DELETE")
        return ret

    def modify(self, hostname, data):
        ret = self.call_url(path=f"v1/objects/hosts/{hostname}", data=self.module.jsonify(data), method="POST")
        return ret

    def diff(self, hostname, data):
        ret = self.call_url(path=f"v1/objects/hosts/{hostname}", method="GET")
        changed = False
        ic_data = ret["data"]["results"][0]
        for key in data["attrs"]:
            if key not in ic_data["attrs"].keys():
                changed = True
            elif data["attrs"][key] != ic_data["attrs"][key]:
                changed = True
        return changed


# ===========================================
# Module execution.
#
def main():
    # use the predefined argument spec for url
    argument_spec = url_argument_spec()
    # add our own arguments
    argument_spec.update(
        state=dict(default="present", choices=["absent", "present"]),
        name=dict(required=True, aliases=["host"]),
        zone=dict(),
        template=dict(),
        check_command=dict(default="hostalive"),
        display_name=dict(),
        ip=dict(),
        variables=dict(type="dict"),
    )

    # Define the main module
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    state = module.params["state"]
    name = module.params["name"]
    zone = module.params["zone"]
    template = []
    if module.params["template"]:
        template = [module.params["template"]]
    check_command = module.params["check_command"]
    ip = module.params["ip"]
    display_name = module.params["display_name"]
    if not display_name:
        display_name = name
    variables = module.params["variables"]

    try:
        icinga = icinga2_api(module=module)
        icinga.check_connection()
    except Exception as e:
        module.fail_json(msg=f"unable to connect to Icinga. Exception message: {e}")

    data = {
        "templates": template,
        "attrs": {
            "address": ip,
            "display_name": display_name,
            "check_command": check_command,
            "zone": zone,
            "vars.made_by": "ansible",
        },
    }
    data["attrs"].update({f"vars.{key}": value for key, value in variables.items()})

    changed = False
    if icinga.exists(name):
        if state == "absent":
            if module.check_mode:
                module.exit_json(changed=True, name=name, data=data)
            else:
                try:
                    ret = icinga.delete(name)
                    if ret["code"] == 200:
                        changed = True
                    else:
                        module.fail_json(msg=f"bad return code ({ret['code']}) deleting host: '{ret['data']}'")
                except Exception as e:
                    module.fail_json(msg=f"exception deleting host: {e}")

        elif icinga.diff(name, data):
            if module.check_mode:
                module.exit_json(changed=False, name=name, data=data)

            # Template attribute is not allowed in modification
            del data["templates"]

            ret = icinga.modify(name, data)

            if ret["code"] == 200:
                changed = True
            else:
                module.fail_json(msg=f"bad return code ({ret['code']}) modifying host: '{ret['data']}'")

    else:
        if state == "present":
            if module.check_mode:
                changed = True
            else:
                try:
                    ret = icinga.create(name, data)
                    if ret["code"] == 200:
                        changed = True
                    else:
                        module.fail_json(msg=f"bad return code ({ret['code']}) creating host: '{ret['data']}'")
                except Exception as e:
                    module.fail_json(msg=f"exception creating host: {e}")

    module.exit_json(changed=changed, name=name, data=data)


# import module snippets
if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The ios_service class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.service import (
    ServiceTemplate,
)


class Service(ResourceModule):
    """
    The ios_service class
    """

    def __init__(self, module):
        super(Service, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="service",
            tmplt=ServiceTemplate(),
        )
        self.parsers = [
            "call_home",
            "compress_config",
            "config",
            "counters",
            "dhcp",
            "disable_ip_fast_frag",
            "exec_callback",
            "exec_wait",
            "hide_telnet_addresses",
            "internal",
            "linenumber",
            "log",
            "log_hidden",
            "nagle",
            "old_slip_prompts",
            "pad",
            "pad_cmns",
            "pad_from_xot",
            "pad_to_xot",
            "password_encryption",
            "password_recovery",
            "private_config_encryption",
            "prompt",
            "pt_vty_logging",
            "scripting",
            "sequence_numbers",
            "slave_coredump",
            "slave_log",
            "tcp_keepalives_in",
            "tcp_keepalives_out",
            "tcp_small_servers",
            "telnet_zeroidle",
            "udp_small_servers",
            "unsupported_transceiver",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state
        """
        wantd = self._service_list_to_dict(self.want)
        haved = self._service_list_to_dict(self.have)

        service_default = {
            "counters": 0,
            "dhcp": True,
            "prompt": True,
            "slave_log": True,
            "password_recovery": True,
        }

        if "private_config_encryption" in haved:
            service_default["private_config_encryption"] = True

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        elif self.state == "deleted":
            wantd = self._service_list_to_dict(service_default)

        # if state is replaced
        elif self.state in ["replaced", "overridden"]:
            wantd = dict_merge(self._service_list_to_dict(service_default), wantd)

        self._compare(want=wantd, have=haved)

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Service network resource.
        """
        self.compare(parsers=self.parsers, want=want, have=have)
        self._compare_lists_attrs(want, have)

    def _compare_lists_attrs(self, want, have):
        """Compare list of dict"""
        i_want = want.get("timestamps", {})
        i_have = have.get("timestamps", {})
        for key, wanting in i_want.items():
            haveing = i_have.pop(key, {})
            if wanting != haveing:
                self.addcmd(wanting, key + "_timestamps", False)
        for key, haveing in i_have.items():
            self.addcmd(haveing, key + "_timestamps", negate=True)

    def _service_list_to_dict(self, data):
        """Convert all list of dicts to dicts of dicts"""
        p_key = {
            "timestamps": "msg",
        }
        tmp_data = deepcopy(data)
        for k, _v in p_key.items():
            if k in tmp_data:
                tmp_data[k] = {str(i[p_key.get(k)]): i for i in tmp_data[k]}
        return tmp_data



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# Copyright: (c) 2023, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema
short_description: Manage schemas
description:
- Manage schemas on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
- Akini Ross (@akinross)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    aliases: [ name ]
  id:
    description:
    - The id of the schema.
    - This parameter is required when the C(schema) needs to be updated.
    type: str
  description:
    description:
    - The description of the schema.
    type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating. Only supported on versions of MSO that are 4.1 or greater.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- Due to restrictions of the MSO REST API this module can only create empty schemas (i.e. schemas without templates) on versions of MSO that are 4.1 or greater.
  Use the M(cisco.mso.mso_schema_template) to automatically create schemas with templates.
seealso:
- module: cisco.mso.mso_schema_site
- module: cisco.mso.mso_schema_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create schema
  cisco.mso.mso_schema:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    state: present
  delegate_to: localhost

- name: Remove schemas
  cisco.mso.mso_schema:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    state: absent

- name: Query a schema
  cisco.mso.mso_schema:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    state: query
  register: query_result

- name: Query all schemas
  cisco.mso.mso_schema:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", aliases=["name"]),
        id=dict(type="str"),
        description=dict(type="str"),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["schema"]],
            ["state", "present", ["schema"]],
        ],
    )

    schema = module.params.get("schema")
    schema_id = module.params.get("id")
    description = module.params.get("description")
    state = module.params.get("state")

    mso = MSOModule(module)
    path = "schemas"

    # Query for existing object(s)
    if schema:
        if schema_id:
            mso.existing = mso.get_obj(path, id=schema_id)
        else:
            mso.existing = mso.get_obj(path, displayName=schema)

        if mso.existing:
            if not schema_id:
                schema_id = mso.existing.get("id")
            path = "schemas/{id}".format(id=schema_id)
    else:
        mso.existing = mso.query_objs(path)

    mso.previous = mso.existing
    if state == "present":
        mso.sanitize(dict(displayName=schema, id=schema_id, description=description), collate=True)
        if mso.existing:
            ops = []
            if mso.existing.get("displayName") != schema:
                ops.append(dict(op="replace", path="/displayName", value=schema))
            if mso.existing.get("description") != description and description is not None:
                ops.append(dict(op="replace", path="/description", value=description))

            if not module.check_mode:
                mso.request(path, method="PATCH", data=ops)
        else:
            if not module.check_mode:
                mso.request(path, method="POST", data=dict(displayName=schema, description=description))
        mso.existing = mso.proposed

    elif state == "absent":
        mso.previous = mso.existing
        if mso.existing:
            if module.check_mode:
                mso.existing = {}
            else:
                mso.existing = mso.request(path, method="DELETE")

    mso.exit_json()


if __name__ == "__main__":
    main()
